//
//  SCSLineChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 5/30/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSLineChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        addSeries()
    }
    
    // MARK: Private Functions

    private func addAxes() {
 
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        addDefaultModifiers()

    }
    
    private func addSeries() {
        
        let dataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        SCSDataManager.putDataInto(dataSeries)
        
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
 
        let ellipsePointMarker = SCIEllipsePointMarker()
        ellipsePointMarker.drawBorder = true
        ellipsePointMarker.fillBrush = SCIBrushSolid(colorCode: 0xFFd6ffd7)
        ellipsePointMarker.height = 5
        ellipsePointMarker.width = 5
        
        let renderSeries = SCIFastLineRenderableSeries()
        renderSeries.dataSeries = dataSeries
        renderSeries.xAxisId = axisXId
        renderSeries.yAxisId = axisYId
        renderSeries.style.pointMarker = ellipsePointMarker
        renderSeries.style.drawPointMarkers = true
        renderSeries.style.linePen = SCIPenSolid(colorCode: 0xFF99EE99, width: 0.7)
        chartSurface.attachRenderableSeries(renderSeries)
        
        
        chartSurface.invalidateElement()
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, Int64(Double(NSEC_PER_SEC) * 5.0) ), dispatch_get_main_queue()) {
            dataSeries.clear()
            self.chartSurface.invalidateElement()
        }
        
    }
    
}
