//
//  SCSFanChartView.swift
//  SciChartSwiftDemo
//
//  Created by Hrybenuik Mykola on 9/22/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSFanChartView: SCSBaseChartView {

    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addDataSeries()
    }
    
    // MARK: Private Methods
    
    private func addAxis() {
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attach(SCSFactoryAxis.createDefaultDateTimeAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attach(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
    }
    
    private func addDataSeries() {
        
        let dataSeries = SCIXyDataSeries(xType:.dateTime, yType:.float)
        dataSeries?.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()

        let xyyDataSeries = SCIXyyDataSeries(xType:.dateTime, yType:.float)
        xyyDataSeries?.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()

        let xyyDataSeries1 = SCIXyyDataSeries(xType:.dateTime, yType:.float)
        xyyDataSeries1?.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let xyyDataSeries2 = SCIXyyDataSeries(xType:.dateTime, yType:.float)
        xyyDataSeries2?.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        self.generatingDataPoints(count: 10) { (result: SCSDataPoint) in
            dataSeries?.appendX(SCIGeneric(result.date), y: SCIGeneric(result.actualValue))
            xyyDataSeries?.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.maxValue), y2: SCIGeneric(result.minValue))
            xyyDataSeries1?.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.value1), y2: SCIGeneric(result.value4))
            xyyDataSeries2?.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.value2), y2: SCIGeneric(result.value3))
        }
      
        let dataRenderSeries = SCIFastLineRenderableSeries()
        dataRenderSeries.dataSeries = dataSeries
        dataRenderSeries.xAxisId = axisXId
        dataRenderSeries.yAxisId = axisYId
        dataRenderSeries.style.linePen = SCIPenSolid(color: UIColor.red, width: 1.0)
        
        chartSurface.attach(createRenderableSeriesWith(dataSeries: xyyDataSeries!))
        chartSurface.attach(createRenderableSeriesWith(dataSeries: xyyDataSeries1!))
        chartSurface.attach(createRenderableSeriesWith(dataSeries: xyyDataSeries2!))
        chartSurface.attach(dataRenderSeries)
        chartSurface.invalidateElement()
        
    }
    
    private func createRenderableSeriesWith(dataSeries: SCIXyyDataSeries) -> SCIBandRenderableSeries {
        let renderebleDataSeries = SCIBandRenderableSeries()
        renderebleDataSeries.style.brush1 = SCIBrushSolid(color: UIColor.init(red: 1.0, green: 0.4, blue: 0.4, alpha: 0.5))
        renderebleDataSeries.style.brush2 = SCIBrushSolid(color: UIColor.init(red: 1.0, green: 0.4, blue: 0.4, alpha: 0.5))
        renderebleDataSeries.style.pen1 = SCIPenSolid(color: UIColor.green, width: 0.5)
        renderebleDataSeries.style.pen2 = SCIPenSolid(color: UIColor.clear, width: 0.5)
        renderebleDataSeries.style.drawPointMarkers = false
        renderebleDataSeries.xAxisId = axisXId
        renderebleDataSeries.yAxisId = axisYId
        renderebleDataSeries.dataSeries = dataSeries
        return renderebleDataSeries;
    }
    
    private func generatingDataPoints(count: Int, handler: (SCSDataPoint) -> Void) {
        var dateTime = NSDate()
        var lastValue: Double = 0.0
        var i: Int = 0
        while i <= count {
            let nextValue = lastValue + SCSDataManager.randomize(min: -0.5, max: 0.5)
            lastValue = nextValue
            dateTime = dateTime.addingTimeInterval(3600*24)
            
            var dataPoint = SCSDataPoint()
            dataPoint.date = dateTime
            dataPoint.actualValue = nextValue
            if i > 4 {
                dataPoint.maxValue = nextValue + (Double(i) - 5) * 0.3
                dataPoint.value1 = nextValue + (Double(i) - 5) * 0.2
                dataPoint.value2 = nextValue + (Double(i) - 5) * 0.1
                dataPoint.value3 = nextValue - (Double(i) - 5) * 0.1
                dataPoint.value4 = nextValue - (Double(i) - 5) * 0.2
                dataPoint.minValue = nextValue - (Double(i) - 5) * 0.3
            }
            
            handler(dataPoint)
            
            i += 1
        }
        
    }
    
    func drawGlToImage() -> UIImage {
        // Draw OpenGL data to an image context
        UIGraphicsBeginImageContext(self.frame.size)

        let bytes = malloc(Int(frame.size.width)*Int(frame.size.height)*4)

        let aContext = UIGraphicsGetCurrentContext()
        glReadPixels(0, 0, GLsizei(Int(frame.size.width)), GLsizei(Int(frame.size.height)), GLenum(GL_RGBA), GLenum(GL_UNSIGNED_BYTE), bytes)
        
        
        let releaseInfo: CGDataProviderReleaseDataCallback = { (info: UnsafeMutableRawPointer?, _, _) -> () in
//            let pointer = COpaquePointer(info)
//            Unmanaged<NSData>.fromOpaque(pointer).release()
            return
        }
        
        let ref = CGDataProvider(dataInfo: nil, data: bytes!, size: Int(frame.size.width)*Int(frame.size.height)*4, releaseData: releaseInfo)
        
        let PreviewBitmapInfo = UInt32(CGImageAlphaInfo.premultipliedFirst.rawValue)
        let renderingIntend = Int32(CGColorRenderingIntent.relativeColorimetric.rawValue)
        
        
        
        
        let iref = CGImage(width: Int(frame.size.width), height: Int(frame.size.height), bitsPerComponent: 8, bitsPerPixel: 32, bytesPerRow: Int(frame.size.width) * 4, space: CGColorSpaceCreateDeviceRGB(), bitmapInfo: CGBitmapInfo(rawValue:PreviewBitmapInfo), provider: ref!, decode: nil, shouldInterpolate: true, intent: CGColorRenderingIntent(rawValue: renderingIntend)!)
        
        aContext!.scaleBy(x: 1.0, y: -1.0)
        aContext!.translateBy(x: 0, y: -self.frame.size.height)
        let im = UIImage(cgImage: iref!)
        UIGraphicsEndImageContext()
        return im
    }
    
}

public struct SCSDataPoint {
    var maxValue = Double.nan
    var minValue = Double.nan
    var value1 = Double.nan
    var value2 = Double.nan
    var value3 = Double.nan
    var value4 = Double.nan
    var actualValue = Double.nan
    var date = NSDate()
    
    
}
